/*
 * Created on Jul 9, 2003
 */
package analysis;

import java.io.IOException;
import java.io.FileInputStream;

import util.Assert;
import util.NoNumberException;
import util.StructuredReader;
import util.Report;

/**
 * @author evans - translated from Selvin's sphere.cc
 */

public abstract class MeasureQuality {
	public abstract int scoreCell(String state, double x, double y, double z);
	public abstract double correctScore ();
		
	public void analyze (String args[]) {
		if (args.length < 1) {
			Report.fatalError("Usage: MeasureQuality <output-file>");
		}

		String filename = args[0];
		FileInputStream datfile = null;

		try {
			datfile = new FileInputStream(filename);
		} catch (IOException ioe) {
			Report.fatalError(
				"Unable to open file for reading: "
					+ filename
					+ ": "
					+ ioe.toString());
		}

		StructuredReader reader = null;

		try {
			reader = new StructuredReader(datfile);
		} catch (IOException ioe) {
			Report.fatalError(
				"Unable to open file for reading: "
					+ filename
					+ ": "
					+ ioe.toString());
		}
	
		try {
			int stepno = 0;

			// Every other iteration, this is read by the readDouble!
			Assert.check(reader.read() == (int) '=');
			Assert.check(reader.read() == (int) '=');
			Assert.check(reader.read() == (int) '=');

			boolean moresteps = true;

			while (moresteps) {
				double correct = correctScore ();
				double score = 0;
				int numCells = 0;
				int goodCells = 0;
				int neutralCells = 0;
				int badCells = 0;

				Assert.check(reader.read() == (int) ' ');
				int readstep = reader.readInt();

				if (stepno != readstep) {
					Assert.badBranch(
						"Step mismatch: " + stepno + " != " + readstep);
				}

				// System.err.println ("Reading step: " + readstep);
				stepno++;

				Assert.check(reader.read() == (int) ' ');
				Assert.check(reader.read() == (int) '=');
				Assert.check(reader.read() == (int) '=');
				Assert.check(reader.read() == (int) '=');
				reader.skipWhitespace();

				while (true) {
					double x;
					String state = null;

					try {
						state = reader.readUntilAny(" ");
						if (state.equals("===")) {
							moresteps = true;
							break;
						}
					} catch (IOException ioe) {
						moresteps = false;
						break;
					}

					Assert.check(reader.read() == (int) ' ');
					x = reader.readDouble();
					Assert.check(reader.read() == (int) ' ');
					double y = reader.readDouble();
					Assert.check(reader.read() == (int) ' ');
					double z = reader.readDouble();
					reader.skipWhitespace();

					double nscore = scoreCell(state, x, y, z);
					
					if (nscore > 0) {
						goodCells++;
					} else if (nscore == 0) {
						;
					} else {
						badCells++;
					}
					
					score += nscore;
					numCells++;
				}

				System.out.println(
					readstep
						+ ": "
						+ numCells
						+ " " // cells score: "
						+ score
						+ " " // weighted: "
						+ (score / correct));
			}
		} catch (IOException ioe) {
			Report.fatalError("Exception: " + ioe);
		} catch (NoNumberException ioe) {
			Report.fatalError("Bad file: " + ioe);
		}
	}
}
