package ga;

import java.io.*;
import parsers.CPA.*;
import simulator.*;
import java.util.*;

import util.Direction;
import util.Assert;

/**
 * @author Brad Zhang
 */

public class FileGen {

	static final int files_num = 2;
	static final int num_states = 14;
	static final int num_goals = 4;
	static final double _mutate_prob = 0.1;

	static public void main(String args[]) throws IOException {
		String cline = "\n";

		for (int i = 1; i < files_num; i++) {
			String name = "tsp";
			name += i;
			name += ".cpa";
			System.out.println("Creating new cpa file: " + name);

			File outputFile = new File(name);
			FileWriter out = new FileWriter(outputFile);

			Genome gen = new Genome(5);
			CPACellProgram cp = generateProgram(gen);
			String cpout = cp.unparse();
			System.out.println(cpout);
			out.write(cpout);
			out.close();
		}
	}

	public static int getGoalsNum() {
		return num_goals;
	}

	static public CPACellProgram generateProgram(Genome gen) {
		// loop through states, constructing parsers.CPA.CellState objects for each state
		// and adding them to cpa

		CPACellProgram cpa = new CPACellProgram();
		int num_states = gen.numberOfStates();

		for (int i = 0; i < num_states; i++) {
			CellState cs = stateDescription(i, gen);
			cpa.addState(cs);
		}
		/*
		if (i >= 2) {
			cd = chemicalDescription();
			cpa.addChemical(cd);
		}
		*/

		return cpa;
	}

	static public CellState stateDescription(int i, Genome gen) {
		int genomeposition = gen.getStatePosition(i);
		String name = "state" + i;
		CPAColor sc = makeColor(gen);

		CellTransitionList ctl = new CellTransitionList();
		CellSecretionNodeList cel = new CellSecretionNodeList();
		CellSecretionNodeList cfl = new CellSecretionNodeList();
		CellSecretionNodeList cdl = new CellSecretionNodeList();
		Vector counters = null;

		cel = emissionList();
		ctl = transitionList();
		return new CellState(name, sc, cel, cfl, cdl, ctl, counters);
	}

	public ChemicalDescription chemicalDescription(Genome gen) {
		Token chem = null;
		chem.image = "chem";
		CPAColor sc = makeColor(gen);
		double spreadrate = -1.0;
		double evaporate = -1.0;
		double diffuserate = -1.0;
		double forcegain = 1.0;
		int diffusionmodel = 0;
		return new ChemicalDescription(
			chem.image,
			sc,
			diffuserate,
			spreadrate,
			evaporate,
			forcegain,
			diffusionmodel);
	}

	static public CPAColor makeColor(Genome gen) {
		int color = gen.readNextInt(2);

		switch (color) {
			case 0 :
				return new CPAColor(
					new LiteralTerm(1.0),
					new LiteralTerm(0.0),
					new LiteralTerm(0.0));
			case 1 :
				return new CPAColor(
					new LiteralTerm(0.0),
					new LiteralTerm(1.0),
					new LiteralTerm(0.0));
			case 2 :
				return new CPAColor(
					new LiteralTerm(0.0),
					new LiteralTerm(0.0),
					new LiteralTerm(1.0));
			case 3 :
				return new CPAColor(
					new LiteralTerm(1.0),
					new LiteralTerm(1.0),
					new LiteralTerm(0.0));
			default :
				Assert.badBranch();
				return null;
		}
	}

	static public CellSecretionNodeList emissionList() {
		CellSecretionNodeList cel = new CellSecretionNodeList();
		CellEmissionNode ce1;
		ce1 = emission();
		cel.add(ce1);
		return cel;
	}

	static public CellEmissionNode emission() {
		String name = "chem";
		Term conc;
		DirectionNode direction = null;
		conc = new LiteralTerm(5);
		return new CellEmissionNode(name, conc, direction);
	}

	public DirectionNode emissionDirection() {
		DirectionNode direction;
		direction = direction();
		return direction;
	}

	public DirectionNode direction() {
		DirectionNode direction;
		Direction basicdirection;
		basicdirection = basicDirection();
		direction = new BasicDirection(basicdirection);
		//		direction = derivedDirection();
		//		direction = new BindingDirection (id.image);
		return direction;
	}

	final public Direction basicDirection() {
		Direction direction;
		direction = Direction.ANY;
		return direction;
		/*		int dir = gen.readNextInt(2);
		
				switch (color) {
					case 0 :
				if () {
					direction = Direction.XP;
				} else if () {
					direction = Direction.XM;
				} else if () {
					direction = Direction.YP;
				} else if () {
					direction = Direction.YM;
				} else if () {
					direction = Direction.ZP;
				} else if () {
					direction = Direction.ZM;
				} else if () {
					direction = Direction.ANY;
				}
				return direction;
		*/
	}

	/* No diffusion yet
		public CellSecretionNodeList diffusionList() {
			  CellSecretionNodeList cel = new CellSecretionNodeList();
			  CellDiffusionNode ce1, ce2;
			  ce1 = diffusion();
			  cel.add(ce1);
			ce2 = diffusion();
			cel.add(ce2);
			return cel;
		}
	*/

	static public CellTransitionList transitionList() {
		CellTransition ct;
		ct = transition();
		CellTransitionList ctl = new CellTransitionList();
		ctl.add(ct);
		return ctl;
	}

	static public CellTransition transition() {
		ChemicalConditionList ccl = new ChemicalConditionList();
//		CellSecretionNodeList cel;
		CellTransition ct = new CellTransition(ccl);
//		cel = emissionList();
//		ct.addCellEmissionList(cel);
		ct.resolve();
		return ct;
	}

	public CellTransition cellTransition(ChemicalConditionList ccl) {
		CellParameters cell1;
		CellParameters cell2 = null;
		DirectionNode direction = cellDivisionDirection();
		cell1 = cellParameters();
		cell2 = cellParameters();
		if (cell2 == null) {
			return new CellTransition(ccl, new CellStateChange(cell1));
		} else {
			return new CellTransition(
				ccl,
				new CellDivision(cell1, cell2, direction));
		}
	}

	public DirectionNode cellDivisionDirection() {
		DirectionNode direction;
		direction = direction();
		return direction;
	}

	public CellParameters cellParameters(){
		Token state = null;
		state.image = "State1";
		Bindings bindings = cellBindings();
		return new CellParameters(state.image, bindings);
	}

	public Bindings cellBindings() {
		Bindings bindings = new Bindings();
		Binding binding;
		binding = binding();
		bindings.addBinding(binding.getIdentifier(), binding.getTerm());
		return bindings;
	}

	public Binding binding(){
		Token lvalue = null;
		lvalue.image = "chem001";
		Term t;
		t = new LiteralTerm (10);
		return new Binding(lvalue.image, t);
	}

	public Vector idList(Token p_id) {
		Vector res = new Vector();
		Token id;
		id = p_id;
		res.add(id.image);
		return res;
	}

	private static String[] parseParams(String s) {
		// Separates s into an array of String
		int numEls = 1;
		int index = 0;

		while ((index = s.indexOf((int) ' ', index)) >= 0) {
			numEls++;
			index++;
		}

		String[] res = new String[numEls];
		index = 0;
		int startindex = 0;
		int elemNo = 0;

		while ((index = s.indexOf((int) ' ', startindex)) >= 0) {
			Assert.check(elemNo < numEls);
			res[elemNo++] = s.substring(startindex, index);
			startindex = index + 1;
		}
		Assert.check(elemNo == numEls - 1);
		// Get the last string
		res[elemNo] = s.substring(startindex);
		return res;
	}

	private static boolean diffFiles(String file1, String file2) {
		boolean diff = false;
		FileInputStream fin1 = null;
		FileInputStream fin2 = null;

		try {
			fin1 = new FileInputStream(file1);
		} catch (FileNotFoundException e) {
			System.err.println("Cannot open file: " + file1 + " - " + e);
		}

		try {
			fin2 = new FileInputStream(file2);
		} catch (FileNotFoundException e) {
			System.err.println("Cannot open file: " + file1 + " - " + e);
		}

		int in1;
		int lineno = 1;

		try {
			while ((in1 = fin1.read()) != -1) {
				while (in1 == '\r') {
					in1 = fin1.read();
				}

				int in2 = fin2.read();

				while (in2 == '\r') {
					in2 = fin2.read();
				}

				if (in1 == in2) {
					if (in1 == '\n') {
						lineno++;
					}
				} else {
					diff = true;
					System.err.println(
						file1
							+ " / "
							+ file2
							+ ":"
							+ lineno
							+ ": "
							+ (char) in1
							+ " / "
							+ (char) in2);
				}
			}
		} catch (IOException e1) {
			System.err.println("Error reading file: " + e1);
			e1.printStackTrace();
		}
		return diff;
	}
}
