/*
 * Created on Nov 11, 2003
 */
package gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Graphics;
import java.util.Enumeration;

import javax.swing.JPanel;

import simulator.Cell;
import simulator.Simulation;
import simulator.SimulationParameters;

class Display extends JPanel {
	static final int hborder = 5;
	static final int wborder = 5;
	
	private Simulation sim;

	Display(Simulation p_sim) {
		sim = p_sim;
		setOpaque(false); // This is necessary to make painting work		
	}

	public void paintComponent(Graphics g) throws RuntimeException {
		g.setColor(Color.WHITE);
		g.fillRect(0, 0, getWidth(), getHeight());

		double maxX, maxY;
		double minX, minY;
		maxX = maxY = minX = minY = 0.0;

		Enumeration enum = sim.getSystemModel().allCells();
		while (enum.hasMoreElements()) {
			double x, y, z;
			Cell cell = (Cell) enum.nextElement();
			x = cell.getX();
			y = cell.getY();
			z = cell.getZ();

			if (true) { // is it on the z plane
				if (x > maxX)
					maxX = x;
				if (y > maxY)
					maxY = y;
				if (x < minX)
					minX = x;
				if (y < minY)
					minY = y;
			}
		}

		double width = maxX - minX;
		double height = maxY - minY;
		double dwidth = getWidth () * 0.95;
		double dheight = getHeight () * 0.95;  
		double wscale = dwidth / width;
		double hscale = dheight / height;
		int wborder = (int) (getWidth () * 0.025);
		int hborder = (int) (getHeight () * 0.025);
		int cellw = Math.max(2, (int) (.8 * wscale));
		int cellh = Math.max(2, (int) (.8 * hscale));
		int cellw2 = (int) ((double) cellw / 2.0);
		int cellh2 = (int) ((double) cellh / 2.0);
		
		int arc = Math.max (cellw/2, 3);
		
		System.err.println ("Drawing cells: width = " + width + " / " + height + " / " + wscale + " / " + hscale + " / " + cellw + " / " + cellh);
		 
		enum = sim.getSystemModel().allCells(); // Need to reset the enum
		while (enum.hasMoreElements()) {
			Cell cell = (Cell) enum.nextElement();
			Color color = cell.getColor().getAwtColor();
			double cellx = cell.getX();
			double celly = cell.getY();
			int actualx = (int) ((cellx - minX) * wscale) + wborder - cellw2;
			int actualy = (int) ((celly - minY) * hscale) + hborder - cellh2;
			
			if (actualx < 0 || actualy < 0 || actualx > getWidth () || actualy > getHeight ()) {
				System.err.println("Drawing cell " + cell + " at " + actualx + ", " + actualy);
			}
			
			g.setColor(color);
			g.fillRoundRect(actualx, actualy, cellw, cellh, arc, arc);
		}
	}
}

public class SimulationWindow2D extends SimulationWindow {
	private Display display;

	SimulationWindow2D(SimulationParameters params) {
		super(params);
		display = new Display(sim);
		Container mainContainer = getContentPane();
		mainContainer.add(display, BorderLayout.CENTER);
	}

	public void refreshDisplay() {
		display.repaint();
	}
}
