/*
 * Created on Nov 11, 2003
 */
package gui;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.GraphicsConfiguration;
import java.util.Enumeration;

import javax.media.j3d.*;
import javax.swing.JPanel;
import javax.vecmath.*;

import com.sun.j3d.utils.geometry.Sphere;
import com.sun.j3d.utils.universe.SimpleUniverse;
import com.sun.j3d.utils.universe.ViewingPlatform;

import simulator.Cell;
import simulator.SimulationParameters;
import util.Position;

/**
 * @author David Evans
 */
public class SimulationWindow3D extends SimulationWindow {
	private BranchGroup lightsGroup;
	private BranchGroup objRoot;
	private TransformGroup objTrans;
	private BranchGroup scene;
	private SimpleUniverse universe = null;

	private double scale = 0.8;

	SimulationWindow3D(SimulationParameters params) {
		super(params);

		GraphicsConfiguration config =
			SimpleUniverse.getPreferredConfiguration();
		Canvas3D canvas = new Canvas3D(config);

		Container mainContainer = getContentPane();

		JPanel pane = new JPanel();
		pane.setLayout(new BorderLayout());
		pane.add(canvas, BorderLayout.CENTER);
		mainContainer.add(pane, BorderLayout.CENTER);

		// Create a simple scene and attach it to the virtual universe
		lightsGroup = null;
		objRoot = null;
		universe = new SimpleUniverse(canvas);
		universe.getViewingPlatform().setNominalViewingTransform();

		scene = new BranchGroup();
		scene.setCapability(BranchGroup.ALLOW_BOUNDS_READ);
		scene.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
		scene.setCapability(BranchGroup.ALLOW_CHILDREN_READ);
		scene.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
		refreshDisplay();
		universe.addBranchGraph(scene);
	}

	public void refreshDisplay() {
		// Create the root of the branch graph
		BranchGroup newRoot = new BranchGroup();
		newRoot.setCapability(BranchGroup.ALLOW_DETACH);

		// Create the transform group node and initialize it to the
		// identity.  Enable the TRANSFORM_WRITE capability so that
		// our behavior code can modify it at runtime.  Add it to the
		// root of the subgraph.
		objTrans = new TransformGroup();
		objTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);

		// System creates the scene graph
		BranchGroup simGraph = getDisplayGraph();
		objTrans.addChild(simGraph);

		TransformGroup transformGroup = new TransformGroup();
		Transform3D trans = new Transform3D();

		if (rotx != 0)
			trans.rotX(Math.toRadians(rotx));
		else if (roty != 0)
			trans.rotY(Math.toRadians(roty));
		else if (rotz != 0)
			trans.rotZ(Math.toRadians(rotz));

		transformGroup.setTransform(trans);
		transformGroup.addChild(objTrans);

		// create the AWTInteractionBehavior 
		AWTInteractionBehavior awtBehaviorX =
			new AWTInteractionBehavior(objTrans, 0);
		buttonRotateX.addActionListener(awtBehaviorX);
		AWTInteractionBehavior awtBehaviorY =
			new AWTInteractionBehavior(objTrans, 1);
		buttonRotateY.addActionListener(awtBehaviorY);
		AWTInteractionBehavior awtBehaviorZ =
			new AWTInteractionBehavior(objTrans, 2);
		buttonRotateZ.addActionListener(awtBehaviorZ);

		// Create a bounds for the background and behaviors
		BoundingSphere bounds =
			new BoundingSphere(new Point3d(0.0, 0.0, 0.0), 100.0);
		awtBehaviorX.setSchedulingBounds(bounds);
		awtBehaviorY.setSchedulingBounds(bounds);
		awtBehaviorZ.setSchedulingBounds(bounds);
		objTrans.addChild(awtBehaviorX);
		objTrans.addChild(awtBehaviorY);
		objTrans.addChild(awtBehaviorZ);

		objTrans = transformGroup;
		newRoot.addChild(objTrans);

		if (lightsGroup == null) {
			BoundingSphere boundsx =
				new BoundingSphere(new Point3d(0.0, 0.0, 0.0), 100.0);
			lightsGroup = new BranchGroup();
			// Set up the background
			Color3f bgColor = new Color3f(1.0f, 1.0f, 1.0f);
			Background bg = new Background(bgColor);
			bg.setApplicationBounds(boundsx);
			lightsGroup.addChild(bg);

			// Set up the global lights
			Color3f lColor1 = new Color3f(0.7f, 0.7f, 0.7f);
			Vector3f lDir1 = new Vector3f(-1.0f, -1.0f, -1.0f);
			Color3f alColor = new Color3f(0.2f, 0.2f, 0.2f);

			AmbientLight aLgt = new AmbientLight(alColor);
			aLgt.setInfluencingBounds(boundsx);
			DirectionalLight lgt1 = new DirectionalLight(lColor1, lDir1);
			lgt1.setInfluencingBounds(boundsx);
			lightsGroup.addChild(aLgt);
			lightsGroup.addChild(lgt1);
			scene.addChild(lightsGroup);
		}

		newRoot.compile();

		if (objRoot != null) {
			scene.removeChild(scene.indexOfChild(objRoot));
		}

		ViewingPlatform myView = universe.getViewingPlatform();
		TransformGroup myViewTransformGroup = myView.getViewPlatformTransform();
		Transform3D myViewTransform = new Transform3D();
		//	System.err.println ("Translating: " + panx + " / " + pany + " / " + panz);
		//  System.err.println ("Rotation: " + rotx + " / " + roty + " / " + rotz);
		myViewTransform.setTranslation(new Vector3d(-panx, -pany, 6.0f - panz));
		myViewTransform.setScale(scale);
		myViewTransformGroup.setTransform(myViewTransform);

		scene.addChild(newRoot);
		objRoot = newRoot;
	}

	public void destroy() {
		universe.removeAllLocales();
	}

	public BranchGroup getDisplayGraph() {
		double maxX = 1,
			maxY = 1,
			maxZ = 1,
			minX = -1,
			minY = -1,
			minZ = -1,
			largest;
		double x, y, z;
		BranchGroup objRoot = new BranchGroup();
		double boxSize = 1.0;

		// Globally used colors
		Color3f black = new Color3f(0.0f, 0.0f, 0.0f);
		Color3f white = new Color3f(1.0f, 1.0f, 1.0f);
		Color3f purple = new Color3f(0.7f, 0.2f, 0.8f);
		Enumeration enum, enums;

		if (sim.getAutoScale()) {
			/* Calculate the box size for display */
			enum = sim.getSystemModel().allCells();
			while (enum.hasMoreElements()) {
				Cell cell = (Cell) enum.nextElement();
				x = cell.getX();
				y = cell.getY();
				z = cell.getZ();
				if (x > maxX)
					maxX = x;
				if (y > maxY)
					maxY = y;
				if (z > maxZ)
					maxZ = z;
				if (x < minX)
					minX = x;
				if (y < minY)
					minY = y;
				if (z < minZ)
					minZ = z;
			}

			/*
			enums = env.getDiffusions().elements();
			while (enums.hasMoreElements()) {
				ChemicalDiffusion cd = (ChemicalDiffusion) enums.nextElement();
				Position Pos = cd.getPosition();
				x = Pos.getX();
				y = Pos.getY();
				z = Pos.getZ();
				if (x > maxX)
					maxX = x;
				if (y > maxY)
					maxY = y;
				if (z > maxZ)
					maxZ = z;
				if (x < minX)
					minX = x;
				if (y < minY)
					minY = y;
				if (z < minZ)
					minZ = z;
			}
			
			maxX = (maxX > -minX) ? maxX : -minX;
			maxY = (maxY > -minY) ? maxY : -minY;
			maxZ = (maxZ > -minZ) ? maxZ : -minZ;
			*/

			largest = (maxX > maxY) ? maxX : maxY;
			//! largest = (largest > maxZ) ? largest : maxZ;
			boxSize = 1.0 / largest;
		} else {
			boxSize = 1.0f;
		}
		enum = sim.getSystemModel().allCells(); // Need to reset the enum
		int curr = 0;
		while (enum.hasMoreElements()) {
			Cell cell = (Cell) enum.nextElement();

			Appearance appearance = new Appearance();
			// Set up the material properties

			Color3f objColor = cell.getColor().getColor3fObject();
			//Color3f objColor = color.getColor3fObject();
			appearance.setMaterial(
				new Material(black, black, objColor, white, 80.0f));
			appearance.setLineAttributes(
				new LineAttributes(2.0f, LineAttributes.PATTERN_SOLID, true));

			TransformGroup objTrans = new TransformGroup();
			Transform3D t = new Transform3D();
			Position cellPos =
				new Position(
					cell.getX() * boxSize,
					cell.getY() * boxSize,
					cell.getZ() * boxSize);
			t.set(cellPos.getVector3dObject());
			objTrans.setTransform(t);
			/*
			  http://www.cs.indiana.edu/csg/links/java3d/html/com/sun/j3d/utils/geometry/Sphere.html
			Node obj = new Box(boxSize/2, boxSize/2, boxSize/2, 
							   Box.GENERATE_NORMALS | Box.GENERATE_TEXTURE_COORDS, appearance);*/

			/*+ (SimRandom.nextDouble() * 0.1)*/
			Node obj = new Sphere((float) ((boxSize) / 3), appearance);
			objTrans.addChild(obj);
			objRoot.addChild(objTrans);

			curr++;
		}

		/*enums = env.getDiffusions().elements();
		while (enums.hasMoreElements()) {
			ChemicalDiffusion cd = (ChemicalDiffusion) enums.nextElement();
			Position Pos = cd.getPosition();
			Appearance appearance2 = new Appearance();
			// Set up the material properties
		
			
			Color3f objColor2 =
				(new Color(cd.getConcentration() / 9,
					cd.getConcentration() / 2,
					cd.getConcentration() / 6))
					.getColor3fObject();
			appearance2.setMaterial(
				new Material(black, black, objColor2, white, 80.0f));
			appearance2.setLineAttributes(
				new LineAttributes(2.0f, LineAttributes.PATTERN_SOLID, true));
		
			PointAttributes p = new PointAttributes(2.0f, true);
			p.setPointAntialiasingEnable(true);
			appearance2.setPointAttributes(p);
		
			TransformGroup objTrans2 = new TransformGroup();
			Transform3D t2 = new Transform3D();
			Position DiffusionPos =
							new Position(
								Pos.getX() * boxSize,
								Pos.getY() * boxSize,
								Pos.getZ() * boxSize);
			t2.set(DiffusionPos.getVector3dObject());
			objTrans2.setTransform(t2);
			/*
			  http://www.cs.indiana.edu/csg/links/java3d/html/com/sun/j3d/utils/geometry/Sphere.html
			Node obj = new Box(boxSize/2, boxSize/2, boxSize/2, 
							   Box.GENERATE_NORMALS | Box.GENERATE_TEXTURE_COORDS, appearance);
		
			Node obj2 = new Sphere((float) boxSize / 2, appearance2);
			//Point3d obj2 = new Point3d(Pos.getX(),Pos.getY(),Pos.getZ());
			objTrans2.addChild(obj2);
			objRoot.addChild(objTrans2);
		}*/

		return objRoot;
	}
}
