package parsers.CPA;

import java.util.*;

import simulator.Cell;
import simulator.CellDiffusion;
import simulator.CellEmission;
import simulator.SimulationParameters;
import util.Assert;
import util.Direction;
import util.InvalidDirectionException;

public class CellSecretionNodeList {
	private Vector secretions;

	public CellSecretionNodeList() {
		secretions = null;
	}

	void add(CellSecretionNode conc) {
		if (secretions == null) {
			secretions = new Vector();
		}

		secretions.addElement(conc);
	}

	public Vector /*[CellEmissionNode]*/
	getList() {
		return secretions;
	}

	public Vector /*[CellSecretion]*/
	getSecretions(Cell c, CPACellProgram prog) {
		if (secretions == null) {
			return null;
		}

		Vector res = new Vector();

		for (int i = 0; i < secretions.size(); i++) {
			CellSecretionNode csn = (CellSecretionNode) secretions.elementAt(i);
			Assert.check(!csn.isSensedDependent());

			if (csn instanceof CellEmissionNode) {
				DirectionNode dirnode = ((CellEmissionNode) csn).getDirection();
				Direction dir;

				if (dirnode == null) {
					dir = Direction.ALL;
				} else {
					try {
						dir = dirnode.eval(null); // no bindings here
					} catch (InvalidDirectionException ide) {
						Assert.badBranch();
						dir = null;
					}
				}

				try {
					res
						.add(new CellEmission(
							csn.getChemical(),
							((CellEmissionNode) csn).getConcentration().eval(
								c,
								null),
					// no cell or bindings
					dir));
				} catch (InvalidDirectionException e) {
					Assert.badBranch ();
				}
			} else if (csn instanceof CellDiffusionNode) {
				String chem = csn.getChemical();
				ChemicalDescription cd = c.getCellProgram().getChemical(chem);

				if (cd == null) {
					util.Report.warning(
						SimulationParameters
							.getSimulationParameters()
							.warnUndeclaredChemicals(),
						"Diffusion is state "
							+ c.getCurrentProgramState()
							+ " uses undeclared chemical: "
							+ chem);

					res.add(
						new CellDiffusion(
							csn.getChemical(),
							((CellDiffusionNode) csn).getConcentration(),
							((CellDiffusionNode) csn).getRate()));
				} else {
					res.add(
						new CellDiffusion(
							csn.getChemical(),
							((CellDiffusionNode) csn).getConcentration(),
							cd.getDiffusionRate(),
							cd.getSpreadRate(),
							cd.getEvaporateRate(),
							cd.getForceGain(),
							cd.getDiffusionModel ()));
				}
			} else {
				Assert.badBranch("Unrecognized cell secretion type: " + csn);
			}
		}

		return res;
	}

	public String toString() {
		if (secretions == null) {
			return "<no secretions>";
		}

		StringBuffer retVal = new StringBuffer("");
		int numEmissions = secretions.size();

		for (int i = 0; i < numEmissions; i++) {
			if (i == numEmissions - 1)
				retVal.append(secretions.elementAt(i).toString());
			else
				retVal.append(secretions.elementAt(i).toString() + ", ");
		}
		return retVal.toString();
	}

	public String unparse() {
		if (secretions == null) {
			return "";
		}

		StringBuffer retVal = new StringBuffer("");
		int numEmissions = secretions.size();

		for (int i = 0; i < numEmissions; i++) {
			if (i == numEmissions - 1)
				retVal.append(secretions.elementAt(i).toString());
			else
				retVal.append(secretions.elementAt(i).toString() + ", ");
		}
		return retVal.toString();
	}

	public int size() {
		return secretions.size();
	}
};
