package parsers.CPA;
import java.util.*;

import simulator.Cell;
import util.Assert;

public class CellState {
	private String state;
	private CPAColor color;
	private CellSecretionNodeList emissions; // emissions
	private CellSecretionNodeList forwards; // forwards
	private CellSecretionNodeList diffusions; // diffusions
	private CellTransitionList transitions;
	private Vector counters;

	public CellState(
		String p_state,
		CPAColor p_color,
		CellSecretionNodeList p_celist,
		CellSecretionNodeList p_cflist,
		CellSecretionNodeList p_cdlist,
		CellTransitionList p_ctlist,
		Vector p_counters) {
		state = p_state;
		color = p_color;
		transitions = p_ctlist;
		forwards = p_cflist;
		emissions = p_celist;
		diffusions = p_cdlist;
		counters = p_counters;
	}

	String getState() {
		return new String(state);
	}

	CPAColor getColor() {
		return color;
	}

	CellTransitionList getCellTransitionList() {
		return transitions;
	}

	Vector getcounters() {
		return counters;
	}

	Vector /*[CellEmission]*/
	getCellEmissionList(Cell c, CPACellProgram prog) {
		if (emissions == null)
			return null;
		else {
			return emissions.getSecretions(c, prog);
		}
	}

	Vector getCellDiffusionList(Cell c, CPACellProgram prog) {
		if (diffusions == null)
			return null;
		else
			return diffusions.getSecretions(c, prog);
	}

	Vector getCellForwardsList(Cell c) {
		Assert.badBranch("Forwards not supported!");
		/*
		if (forwards == null)
			return null;
		else
			return forwards.getList();
			*/
		return null;
	}

	public String toString() {
		StringBuffer retVal = new StringBuffer("");

		retVal.append("state " + state + " {\n");

		if (color != null) {
			retVal.append("\tcolor " + color.toString() + "\n");
		}

		if (emissions != null) {
			retVal.append("\temits" + emissions.toString() + "\n");
		}

		if (forwards != null) {
			retVal.append("\tforwards " + forwards.toString() + "\n");
		}

		if (diffusions != null) {
			retVal.append("\tdiffuses " + diffusions.toString() + "\n");
		}

		/*		if (counters != null) {
					retVal.append("\tcounters " + counters.toString() + "\n");
				}
		*/
		if (transitions != null) {
			retVal.append("\ttransitions\n" + transitions.toString());
		}
		retVal.append("}\n\n");
		return retVal.toString();
	}

	public String unparse() {
		StringBuffer retVal = new StringBuffer("");

		retVal.append("state " + state + " {\n");

		if (color != null) {
			retVal.append("\tcolor " + color.unparse() + "\n");
		}

		if (emissions != null) {
			retVal.append("\temits " + emissions.unparse() + "\n");
		}

		if (forwards != null) {
			retVal.append("\tforwards " + forwards.unparse() + "\n");
		}

		if (diffusions != null) {
			retVal.append("\tdiffuses " + diffusions.unparse() + "\n");
		}

		/*				if (counters != null) {
							retVal.append("\tcounters " + counters.unparse() + "\n");
						}
		*/
		if (transitions != null) {
			retVal.append("\ttransitions\n" + transitions.unparse());
		}

		retVal.append("}\n\n");

		return retVal.toString();

	}
}
