package parsers.CPA;

import simulator.Bindings;
import util.*;
import java.util.Vector;

public class CellTransition {
	private ChemicalConditionList condition;
	private CellSecretionNodeList emits;
	private CellSecretionNodeList stores;
	private CellSecretionNodeList diffuses;
	private boolean isCellDivision;
	private boolean isCellDeath;
	private CellDivision division = null;
	private CellStateChange stateChange = null;

	public CellTransition(ChemicalConditionList p_CCList, CellDivision p_Division) {
		isCellDeath = false;
		isCellDivision = true;
		condition = p_CCList;
		division = p_Division;
	}

	public CellTransition(ChemicalConditionList p_CCList, CellStateChange p_Change) {
		isCellDeath = false;
		isCellDivision = false;
		condition = p_CCList;
		stateChange = p_Change;
	}

	public CellTransition(ChemicalConditionList p_CCList) {
		isCellDeath = true;
		isCellDivision = false;
		condition = p_CCList;
	}

	public void resolve () {
		// Check the variables used in the transition rules.
		// Every variable used in the secretions must be bound.
	    Bindings vars = condition.getVariables ();				
	}
	
	ChemicalConditionList getChemicalConditionList() {
		return condition;
	}

	CellDivision getCellDivision() {
		return division;
	}

	CellStateChange getCellStateChange() {
		return stateChange;
	}

	void addCellEmissionList(CellSecretionNodeList p_CEList) {
		emits = p_CEList;
	}

	void addCellStorageList(CellSecretionNodeList p_CSList) {
		stores = p_CSList;
	}

	void addCellDiffusionList(CellSecretionNodeList p_CDList) {
		diffuses = p_CDList;
	}

	boolean hasEmissions() {
		return emits != null && emits.size() > 0;
	}

	CellSecretionNodeList getCellEmissionList() {
		return emits;
	}

	boolean hasStorage() {
		return stores != null && stores.size() > 0;
	}

	CellSecretionNodeList getCellStorageList() {
		if (stores != null) {
			return stores;
		} else {
			Assert.badBranch("Null cell storage list!");
			return null;
		}
	}

	boolean hasDiffusions() {
		return diffuses != null && diffuses.size() > 0;
	}

	CellSecretionNodeList getCellDiffusionList() {
		return diffuses;
	}

	public void addChemicals(Vector vec) {
		if (hasEmissions()) {
			vec.add(getCellEmissionList().getList());
		} else {
			vec.add(null);
		}

		if (hasStorage()) {
			vec.add(getCellStorageList().getList());
		} else {
			vec.add(null);
		}

		if (hasDiffusions()) {
			vec.add(getCellDiffusionList().getList());
		} else {
			vec.add(null);
		}
	}

	public String toString() {
		if (isCellDeath) {
			return condition.toString()
				+ " -> die\n"
				+ (hasEmissions() ? emits.toString() : "")
				+ (hasStorage() ? stores.toString() : "");
		} else if (isCellDivision) {
			return condition.toString()
				+ " -> "
				+ division.toString()
				+ (hasEmissions() ? emits.toString() : "")
				+ (hasStorage() ? stores.toString() : "");
		} else {
			return condition.toString()
				+ " -> "
				+ stateChange.toString()
				+ ";"
				+ (hasEmissions() ? emits.toString() : "")
				+ (hasStorage() ? stores.toString() : "");
		}
	}

	public boolean isDeath() {
		return isCellDeath;
	}
	
	public boolean isDivision() {
		return isCellDivision;
	}
	
	public boolean isStateChange() {
		return !isCellDeath && !isCellDivision;
	}
}
