/*
 * Created on Jun 26, 2003
 */
package parsers.CPA;
import util.*;
import simulator.Bindings;
import simulator.Cell;

/**
 * @author David Evans
 */

public class OpTerm extends Term {
	private Term left, right;
	private Token op;

	public OpTerm(Term p_left, Token p_op, Term p_right) {
		super();
		left = p_left;
		op = p_op;
		right = p_right;
	}

	public Direction evalDir (Cell c, Bindings b) throws InvalidDirectionException {
		Direction lval = left.evalDir (c, b);
		Direction rval = right.evalDir (c, b);

		switch (op.kind) {
			case CPAParserConstants.MINUS:
				return lval.minus (rval);
			case CPAParserConstants.PLUS :
				return lval.plus (rval);
			case CPAParserConstants.TIMES :
			case CPAParserConstants.DIVIDE :
			default :
				Assert.badBranch("Bad op: " + op);
		}
		
		Assert.badBranch ("eval");
		return null;	
	}
	
	public double eval (Cell c, Bindings b) throws InvalidDirectionException {
		double lval = left.eval (c, b);
		double rval = right.eval (c, b);

		switch (op.kind) {
			case CPAParserConstants.MINUS:
				Report.message (toString () + " for " + c + "==> " + (lval - rval));
				return lval - rval;
			case CPAParserConstants.PLUS :
				return lval + rval;
			case CPAParserConstants.TIMES :
				return lval * rval;
			case CPAParserConstants.DIVIDE :
				return lval / rval;
			default :
				Assert.badBranch("Bad op: " + op);
		}
		
		Assert.badBranch ("eval");
		return 0.0;
	}

	public String toString () {
		return left.toString () + " " + op.image + " " + right.toString ();
		
	}
}
