
/*
 * Created on Jul 1, 2003
 */
package simulator;

import java.util.Enumeration;

import util.Direction;
import util.DirectionSet;
import util.StringTable;

/**
 * @author evans
 */
public class Bindings {

	// Bindings is a variable -> value mapping associated with a cell action.
	private StringTable rep;
	
	public Bindings () {
		rep = new StringTable ();
	}

	public Bindings(String var, Direction dir) {
		rep = new StringTable ();
		addBinding (var, dir);
	}
	
	public void addAll (Bindings b) {
		if (b != null) {
			rep.addAll (b.rep);
		}
	}
	
	public void addBinding (String var, Object value) {
		rep.add (var, value);
	}

	public String getKey(int bindingno) {
		return rep.getKey (bindingno);
	}

	public Enumeration getKeys() {
		return rep.keys ();
	}

	public Object getValue (int i) {
		return rep.getValue(i);
	}

	public boolean isEmpty() {
		return rep.size() == 0;
	}


//	Requires: None
//	Modifies: None
//	Ensures: If var is in Bindings returns the value related to var
//				otherwise returns null.
	public Object lookup(String var) {
		return rep.lookup (var);
	}

	public void selectDirections() {
		// For each binding that is a set, select one direction from that set
		for (Enumeration vars = rep.keys(); vars.hasMoreElements (); ) {
			String key = (String) vars.nextElement ();
			Object value = rep.lookup (key);
			
			if (value instanceof DirectionSet) {
				DirectionSet dirs = (DirectionSet) value;
				Direction dir = dirs.chooseRandomly ();
				
				updateBinding (key, dir);
			}
		}
	}

	public int size() {
		return rep.size ();
	}
	
	public String toString () {
		return /* "<bindings: " + rep.toString () + ">" */ rep.toString ();
	}

	public void updateBinding(String key, Object dir) {
		rep.update (key, dir);		
	}
}
