/*
 * Created on Jul 1, 2003
 */
package simulator;
import sysmodels.ChemicalDiffusion;

/**
 * @author David Evans
 */

public class CellDiffusion extends CellSecretion {

	private static final double DEFAULT_SPREAD_RATE = 100.0;
	private static final double DEFAULT_EVAPORATE_RATE = 0.1;
	private static final double DEFAULT_FORCE_GAIN = 1.0;
	
	private double diffuserate;
	private double spreadrate;
	private double evaporate;
	private double forcegain;
	private int diffusemodel;
	
	public CellDiffusion(String p_chemical, double p_conc, double p_rate) {
		super (p_chemical, p_conc);
		diffuserate = p_rate;
		spreadrate = DEFAULT_SPREAD_RATE;
		evaporate = DEFAULT_EVAPORATE_RATE;
		forcegain = DEFAULT_FORCE_GAIN;
		diffusemodel = ChemicalDiffusion.LINEAR;
	}

	public CellDiffusion(String p_chemical, double p_conc, double p_diffuserate,
	double p_spreadrate, double p_evaporate, double p_forcegain, int p_diffusemodel) {
		super (p_chemical, p_conc);
		diffuserate = p_diffuserate;
		spreadrate = p_spreadrate;
		evaporate = p_evaporate;
		forcegain = p_forcegain;
		diffusemodel = p_diffusemodel;
	}

	public double getSpreadRate () { // time it takes gradient to travel one unit 
		return spreadrate;
	}
	
	public double getDiffuseRate () { // slope of gradient
		return diffuserate;
	}
	
	public double getEvaporateRate () {
		return evaporate;
	}
	
	public double getForceGain () {
		return forcegain;
	}
	
	public String toString () {
		String res = "diffuses (" + getChemical () + ", " + getConcentration ();
		
		if (diffuserate != 1.0) {
			res += " at " + diffuserate;
		}
		
		res += " spread: " + spreadrate + " evap: " + evaporate;
		res += ")";
		
		return res;
	}

	public int getDiffusionModel() {
		return diffusemodel;
	}
}
