package simulator;

import java.util.*;

class SimToken
{
	final static int SIM_EOF = 0;
	final static int SIM_INSTRUCTION = 1;
	final static int SIM_CONFIG = 2;
	final static int SIM_ERROR = 3;

	private int simTokenType;
	private Vector tokenComponents;

	public SimToken()
	{
		simTokenType = SIM_EOF;
	}	

	public SimToken(int instructionNumber, Vector instruction)
	{
		simTokenType = SIM_INSTRUCTION;
		tokenComponents = new Vector();
		tokenComponents.add(new Integer(instructionNumber));
		tokenComponents.add(instruction);
	}	

	public SimToken(String configParam, String configValue, Vector configOptions)
	{
		simTokenType = SIM_CONFIG;
		tokenComponents = new Vector();
		tokenComponents.add(configParam);
		tokenComponents.add(configValue);
		tokenComponents.add(configOptions);
	}
	
	public SimToken(int lineNumber)
	{
		simTokenType = SIM_ERROR;
		tokenComponents = new Vector();
		tokenComponents.add(new Integer(lineNumber));
	}

	public int getType()
	{
		return simTokenType;
	}

	public int getInstructionNumber()
	{
		if(getType() == SIM_INSTRUCTION)
			return ((Integer)tokenComponents.elementAt(0)).intValue();
		else
			return -1;
	}

	public Vector getInstruction()
	{
		if(getType() == SIM_INSTRUCTION)
			return (Vector)tokenComponents.elementAt(1);
		else
			return new Vector();
	}

	public String getConfigParam()
	{
		if(getType() == SIM_CONFIG)
			return (String)tokenComponents.elementAt(0);
		else
			return new String("");
	}

	public String getConfigValue()
	{
		if(getType() == SIM_CONFIG)
			return (String)tokenComponents.elementAt(1);
		else
			return new String("");
	}

	public Vector getConfigOptions()
	{
		if(getType() == SIM_CONFIG)
			return (Vector)tokenComponents.elementAt(2);
		else
			return new Vector();
	}

	public int getErrorLineNumber()
	{
		if(getType() == SIM_ERROR)
			return ((Integer)tokenComponents.elementAt(0)).intValue();
		else
			return 0;
	}
	
	public String toString()
	{
		StringBuffer retVal = new StringBuffer("");

		if(simTokenType == SIM_EOF)
		{
			retVal.append(new String("SIM_EOF: "));
		}
		else if(simTokenType == SIM_CONFIG)
		{
			int num = 0;
			num = getConfigOptions().size();
			
			retVal.append(new String("SIM_CONFIG: "));
			retVal.append(getConfigParam() + " ");
			retVal.append(getConfigValue() + " ");
			for(int i=0; i<num; i++)
			{
				retVal.append(getConfigOptions().elementAt(i).toString() + " ");
			}
		}
		else if(simTokenType == SIM_INSTRUCTION)
		{
			int num = 0;
			num = getInstruction().size();
			
			retVal.append(new String("SIM_INSTRUCTION: "));
			retVal.append(getInstructionNumber() + " ");
			for(int i=0; i<num; i++)
			{
				retVal.append(getInstruction().elementAt(i).toString() + " ");
			}
		}
		else if(simTokenType == SIM_ERROR)
		{
			retVal.append(new String("SIM_ERROR: "));
		}

		return retVal.toString();
	}
}
