package sysmodels;

import util.*;

/** This class was previously named CellDiffusion, but renamed ChemicalDiffusion to avoid confusion
 ** with the simulator classes!
 **/

// CellDifussion is one object in vector CellDiffusionList used in env, 
// when resolveDiffusion, env will calculate the conc based, on the CellDiffusionlist
// in all the previous models, CellEmission is used as a datatype that represents bothe CellEmission and CellDiffusion
// now, CellEmission and CellDiffusion are seperate datatypes, they are interchangeable, however, CellDiffusion
// has no use of direction and operation because diffusion is in all direction.
public class ChemicalDiffusion {
	private Position pos; // was x, y, z
	private String chemical;
	private double concentration;
	private double diffuserate; // was factor
	private double spreadrate;
	private double evaporate;
	private int step;
	private int diffusemodel;
	
	public final static int LINEAR = 0;
	public final static int EXPONENTIAL = 1;
	public final static int BELLSHAPE = 2;
	public final static int SUDDENRISE = 3;
	public final static int SUDDENDROP = 4;
	public final static int OSCILLATION = 5;
	
	
	public int getStep(){
		return step;
	}
	
	public ChemicalDiffusion(
		Position p_pos,
		String p_chem,
		double p_concentration,
		double p_diffuserate,
		double p_spreadrate,
		double p_evaporate,
		int p_diffusemodel,
		int p_step) {
		pos = p_pos;
		chemical = p_chem;
		concentration = p_concentration;
		diffuserate = p_diffuserate;
		spreadrate = p_spreadrate;
		evaporate = p_evaporate;
		diffusemodel = p_diffusemodel;
		step = p_step;
	}

	/*CellDiffusion(double X, double Y, double Z,String p_chem, double p_factor){
		 x=X;y=Y;z=Z;
		ce=new CellEmission(p_chem);
		factor = p_factor;
	}*/

	public ChemicalDiffusion(ChemicalDiffusion cd) {
		pos = cd.pos;
		chemical = cd.getChemical();
		concentration = cd.getConcentration();		
		diffuserate = cd.getDiffuseRate();
		spreadrate = cd.getSpreadRate();
		evaporate = cd.getEvaporateRate ();
		diffusemodel = cd.getDiffusionModel ();
		step = cd.getStep();
	}

	/*CellDiffusion(double X, double Y, double Z, CellEmission ce, double p_factor){
		x=X;y=Y;z=Z;
		ce=new CellEmission(ce);
		factor = p_factor;
	}*/

	/*CellDiffusion (double X, double Y, double Z, String p_chem, String p_operator, double p_conc, Direction p_direction, double p_factor)
	{ x=X;y=Y;z=Z;
	ce=new CellEmission(p_chem,p_operator,p_conc,p_direction);
	factor = p_factor;}*/

	public Position getPosition() {
		return pos;
	}

	public String getChemical() {
		return chemical;
	}

	public double getConcentration() {
		return concentration;
	}

	public double getDiffuseRate() {
		return diffuserate;
	}

	public int getDiffusionModel () {
		return diffusemodel; 
	}
	
	public double getSpreadRate() {
		return spreadrate;
	}
	
	public double getEvaporateRate() {
		return evaporate;
	}
	public String toString() {
		return "<diffusion at " + pos +": " + chemical + " / " + concentration + ", " + diffuserate + ", " + diffusemodel + ">";
	}
}
