package sysmodels;

//sites for  vector information in 2d and 3d.
//http://em-ntserver.unl.edu/Math/mathweb/vectors/vectors.html#vec1
//http://www.jtaylor1142001.net/calcjat/Solutions/VDotProduct/VDPSVP3D.htm

import util.*;
import java.lang.Math;

public class EVector {
	Position startP;
	Position endP;

	public Position getStartPoint() {
		return startP;
	}

	public Position getEndPoint() {
		return endP;
	}

	public void setStartPoint(Position p_startP) {
		startP = new Position(p_startP);
	}

	public void setEndPoint(Position p_endP) {
		endP = new Position(p_endP);
	}

	public EVector() {
		startP = new Position(null);
		endP = new Position(null);
	}

	public EVector(Position p_startP, Position p_endP) {
		startP = new Position(p_startP);
		endP = new Position(p_endP);
	}

	public EVector(EVector p_vec) {
		startP = p_vec.getStartPoint();
		endP = p_vec.getEndPoint();
	}

	public boolean isAdjacent(EVector p_vec) {
		if (this.equals(p_vec)) {
			return false;
		} else {
			Position p1 = p_vec.getStartPoint();
			Position p2 = p_vec.getEndPoint();
			if (startP.equals(p1)
				|| endP.equals(p1)
				|| startP.equals(p2)
				|| endP.equals(p2)) {
				return true;
			} else {
				return false;
			}
		}
	}

	public double getLength() {
		double dist = startP.distance(endP);
		return dist;
	}

	//p_vec and EVector are adjacent at startP. return true if p_vec is accelerating EVector, false otherwise
	public boolean isAccelerate(EVector p_vec) {
		double dist1 = p_vec.getLength();
		double dist2 = this.getLength();
		double dist3 = endP.distance(p_vec.getEndPoint());
		if ((dist1 * dist1 + dist2 * dist2) > (dist3 * dist3)) {
			return true;
		} else {
			return false;
		}
	}
	//p_vec and EVector are adjacent. projection of p_vec onto EVector. return type is a double
	public double getProjectionLength(EVector p_vec) {
		Position p1 = p_vec.getStartPoint();
		Position p2 = p_vec.getEndPoint();

		//p_vec = x1 i + y1 j + z1 k
		double x1 = p2.getX() - p1.getX();
		double y1 = p2.getY() - p1.getY();
		double z1 = p2.getZ() - p1.getZ();

		//EVector = x2 i + y2 j + z2 k
		double x2 = endP.getX() - startP.getX();
		double y2 = endP.getY() - startP.getY();
		double z2 = endP.getZ() - startP.getZ();

		//scalar proj of p_vec onto EVector 
		double dist =
			(x1 * x2 + y1 * y2 + z1 * z2) / Math.sqrt(x2 * x2 + y2 * y2 + z2 * z2);
		return dist;
	}

	

	public boolean equals(EVector p_vec) {
		Position p1 = p_vec.getStartPoint();
		Position p2 = p_vec.getEndPoint();
		if ((startP.equals(p1) && endP.equals(p2))
			|| (startP.equals(p2) && endP.equals(p1)))
			return true;
		else
			return false;
	}

}