package sysmodels;
import util.*;
import java.util.*;

// diffusion is stored as a list in env
// when look up the conc in enviroment, calculate the conc resolved from the diffusion list based on
// current model: conc = conc_at_where_diffusion_occurced - distance + factor //factor is a variation of controlled models
// need improve: better models for calculating conc.

public class Environment {
	private Vector celldiffusionList;

	//force 
	private EVector ForceVector;
	private Vector ForceVectorList;

	public Environment() {
		celldiffusionList = new Vector();
	}

	public Environment(EVector p_vector) {
		celldiffusionList = new Vector();
		ForceVector = new EVector(p_vector);

	}

	public Environment(Vector p_vectorList) {
		celldiffusionList = new Vector();
		//ForceVectorList.
		if (p_vectorList != null) {
			ForceVectorList = new Vector();
		}

		Enumeration enum = p_vectorList.elements();
		while (enum.hasMoreElements()) {
			ForceVectorList.add((EVector) enum.nextElement());
		}
	}

	// added this to replace force vector
	public void setForceVector(EVector p_vector) {
		ForceVector = new EVector(p_vector);
	}

	public void clearDiffusions() {
		Report.message("Clearing diffusions");
		celldiffusionList.clear();
	}

	public void add(ChemicalDiffusion cd) {
		celldiffusionList.addElement(cd);
		Report.message("Adding diffusion: " + cd);
	}

	public Vector getDiffusions() {
		return celldiffusionList;
	}

	void setList(Vector cdl) { //! this is never used
		if (celldiffusionList != null) {
			celldiffusionList = new Vector();
		}

		Enumeration enum = cdl.elements();
		while (enum.hasMoreElements()) {
			add((ChemicalDiffusion) enum.nextElement());
		}
	} //better not be used at all

	void appendList(Vector cdl) {
		Enumeration enum = cdl.elements();
		while (enum.hasMoreElements()) {
			add((ChemicalDiffusion) enum.nextElement());
		}
	} //better not be used at all

	public double findConcentration(String chem, Position pos, int step) {
		double conc = 0.0;
		double dist;

		Enumeration enums = celldiffusionList.elements();
		while (enums.hasMoreElements()) {
			ChemicalDiffusion cd = (ChemicalDiffusion) enums.nextElement();
			//System.out.println("step"+cd.getStep()+"s "+step);
			//check if the right chemical,
			if (chem.equals(cd.getChemical())) {
				//CellEmission ce = cd.getCellEmission();
				//conc = ce.getConcentration ();
				double newconc = cd.getConcentration();
				dist = cd.getPosition().distance(pos);
				Report.message(
					5,
					"findConcentration: "
						+ chem
						+ " at "
						+ pos
						+ ": "
						+ newconc);
				Report.message(5, "Diffusion: " + cd);
				try {

					/*
					 //<---------------------------------multi forces:---------------------------------> 
					 Enumeration enumers = ForceVectorList.elements();
					while (enumers.hasMoreElements()) {
						EVector FVector = (EVector) enumers.nextElement();
						FVector.setStartPoint(cd.getPosition());
						double ProjectionLength =
							vec.getProjectionLength(ForceVector);
						newconc = newconc - dist * cd.getFactor();
						//linear model.	
							newconc = (1 + ProjectionLength) * newconc;
						
					}*/

					//<-------newconc = newconc - (dist * cd.getDiffuseRate ()); //linear model.------>
					/* <-------------------------------------Time model-------------------------------> 
					  
					 start at certain time step with full conc, conc drops graduately corresponding to time steps.
					 
					 if (timeS < spreadtime) {
					
																	 newconc = 0.0;//timeS / spreadtime * newconc;
						
																 } else if (timeS >= spreadtime) {
																	 newconc = newconc - timeS * evaporrate;
																 }*/
					double diffuserate = cd.getDiffuseRate();
					int diffusemodel = cd.getDiffusionModel();
					double spreadrate = cd.getSpreadRate();
					// Report.message(0,"diffusemodel: "+diffusemodel);
					// Report.message(0,"cd: "+cd);
					double evaporrate = cd.getEvaporateRate();
					int timeS = step - cd.getStep();
					double spreadtime = dist / spreadrate;
					//double evaportime = dist / evaporrate;

					switch (diffusemodel) {
						case ChemicalDiffusion.LINEAR :
							newconc = newconc - dist;
							if (newconc < 0.0) {
								newconc = 0;
							}
							break;
						case ChemicalDiffusion.BELLSHAPE :
							{

								//<------------------------bell-shape model:-------------------------------------->
								// f(x) = 1/(sqrt(2*pi)*sigma)*exp(-pow((x-mu),2.0)/(2*sigma*sigma))
								// center: mu; start: mu-2*sigma and end: mu+2*sigma; maxheight: .399/sigma
								// shrinkfactor >1 the bell-shape shrinks with same height and same center.
								//ex. giant & eve stripes 
								//Report.message(0,"sudden-drop executed");
								double mu = evaporrate;
								double shrinkfactor = diffuserate;
								double sigma = .399 / newconc;
								newconc =
									1
										/ (Math.sqrt(2 * 3.141593) * sigma)
										* Math.exp(
											-shrinkfactor
												* Math.pow((dist - mu), 2.0)
												/ (2 * sigma * sigma));
								break;
							}
						case ChemicalDiffusion.SUDDENDROP :
							{

								//<-----------------------sudden-drop model-------------------------------------->
								//conc stays constant before mu, conc then drops as an right-side bell-shape curve
								//ex. huncback
								double mu = evaporrate;
								double shrinkfactor = diffuserate;
								double sigma = .399 / newconc;
								Report.message("sudden-drop executed");
								if (dist <= mu) {
								} //conc stays same until certain point
								else {

									newconc =
										1
											/ (Math.sqrt(2 * 3.141593) * sigma)
											* Math.exp(
												-shrinkfactor
													* Math.pow((dist - mu), 2.0)
													/ (2 * sigma * sigma));
								}
								break;
							}
						case ChemicalDiffusion.SUDDENRISE :
							{
								//<------------------------sudden rise model----------------------------------------->
								//conc raises from (mu-2*sigma, 0) as an left-side bell-shape curve until at point mu
								//conc reaches its original threshold and conc stays constant after mu. 
								//ex. kruppel.
								double mu = evaporrate;
								double shrinkfactor = diffuserate;
								double sigma = .399 / newconc;

								if (dist > mu) {
								} //conc stays same until certain point
								else {
									newconc =
										1
											/ (Math.sqrt(2 * 3.141593) * sigma)
											* Math.exp(
												-shrinkfactor
													* Math.pow((dist - mu), 2.0)
													/ (2 * sigma * sigma));
								}
								break;
							}
						case ChemicalDiffusion.OSCILLATION :
							//<--------------------------oscillation model------------------------------------>
							//conc is a function of sin(dist), start at(0, 0) if evaporrate=1, start at(0,maxheigh) if 
							//evaporrate=4
							// and maxheight = original concentration.
							//diffuserate>1, sin() is shrinked.  
							//used for creating strips
							newconc =
								newconc
									+ newconc
										* Math.sin(
											diffuserate * dist - evaporrate);
							break;
						case ChemicalDiffusion.EXPONENTIAL :
							//<-------------------exponential model:------------------------------------------>
							// start at dist=0, conc=newconc, diffusion happens fast so need high concentration.
							//the smaller the diffuserate, the slower the diffusion rate (highter conc).
							//ex. bicoid
							newconc = newconc * Math.exp(-dist * diffuserate);
							break;
						default :
							Assert.badBranch(
								"Unrecognized diffusion model: "
									+ diffusemodel);

					}

					if (ForceVector != null) {
						//Report.setReportLevel(5);
						//Report.message(
						//		"Adjust for force vector: "
						//			+ newconc
						//			+ " at "
						//			+ pos);
						//a vector starts from diffusion location to pos(where conc to be calculated )
						EVector vec = new EVector(cd.getPosition(), pos);
						//projection length of ForceVector onto vec.
						ForceVector.setStartPoint(cd.getPosition());
						double ProjectionLength =
							vec.getProjectionLength(ForceVector);

						//Report.message(
						//	"Projection length: " + ProjectionLength);
						//				newconc = newconc * Math.exp(-dist*cd.getFactor());

						newconc = (1 + ProjectionLength) * newconc;

						//Report.message ("New concentration: " + newconc);
						//Report.setReportLevel(0);
					}
					if (newconc > conc) {
						conc = newconc;
						Report.message(5, "Setting concentration: " + conc);
					}
				} catch (NullPointerException e) {
					Report.fatalError("Null pointer exception: " + e);
				}

			}
		}
		Report.message(5, "Returning concentration: " + conc);
		return conc;
	}

}
