/*
 * Edited on Jun 10, 2003
 *
 * To change the template for this generated file go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
 
 package sysmodels.dnabiology;

/**
 * @editor Brian
 * When performCellDivision, the son cell has inherited dna
 */

import java.util.*;

import simulator.*;
import util.*;
import cellmodels.dnacell.*;

public class DNABiology extends sysmodels.standard.StandardModel {
	public DNABiology(Simulation p_sim) {
			super(p_sim);
			System.err.println ("CREATING DNABIOLOGY!");
	}
	
	protected void updateStatistics () {
		super.updateStatistics ();
		statistics.setValue("Original DNA Counter Value",
							getSimulation().getDNADefault());
	}

	protected Cell getNewCell (Vector states, int programNum) {	 
		Cell cell = getSimulation().createCell (numberOfCells (), this, cellProgram, states, programNum);
		if (getSimulation().getDNADefault()>0) {  
			dna.setDefault ( getSimulation().getDNADefault() );
		}
		((DNACell) cell).init (new dna ());
		IDMappings.add(cell);
		return cell;
	}

	protected Cell getNewCell (Vector states, dna olddna, int programNum) {
		Cell cell = getSimulation().createCell (numberOfCells(), this, cellProgram, states, programNum);
		((DNACell) cell).init (olddna);
		IDMappings.add(cell);
		return cell;
	}

	private dna getCellDNA (Cell cd) {
		return ((DNACell) cd).getdna ();
	}
	
	public void performCellDivision (
		Cell cell,
		String state1,
		String state2,
		Direction divisionDir) {
		Cell cell1 = cell;
		Cell cell2 = null;

		Report(
			"Performing division: "
				+ divisionDir
				+ " from "
				+ state1
				+ " / "
				+ state2);

		System.err.println ("DNABiology Performing division: " + divisionDir + " from " + state1 + " / " + state2);
		System.err.println ("Counter: " + ((DNACell) cell1).getdna ());
		
		//check dna
		if (!getCellDNA (cell1).checkdna() ) {
			return;
		}
	
		// added dna	
		dna olddna = getCellDNA (cell1);		
		
		//cell2 = getNewCell(state2, olddna);

		if (cell1 == null) {
			Assert.badBranch("Non-existant cell requested cell division?");
		}
		if (cell2 == null) {
			Assert.badBranch("Not enough Memory");
		}
		if (divisionDir == null) {
			Assert.badBranch(
				"Direction for division null: " + state1 + " / " + state2);
		}

		((DNACell) cell2).setdna (dna.dnaCopyForOneStep (getCellDNA (cell1)));

		//oldCell dna change or not?
		//cd.setdna(....)

		Position pstart = cell.getPosition();
		Position pnew = pstart.offset(divisionDir);
		cell2.setPosition(pnew);

		if (displaceCells(cell1, cell2)) {
			// cell2 is added, change cell1 state
			cell.setCurrentProgramState(state1);
		} else {
			// No space for cell2
			killCell (cell2) ;
		}
	}

	static class DNABiologyFactory extends SystemFactory {
		public SystemModel create(Simulation p_sim, String modelOptions) {
			DNABiology sb = new DNABiology(p_sim);

			Vector vec = new Vector();
			vec.add(modelOptions);
			sb.init(vec);

			return sb;
		}
	}

	static {
		Simulation.sysModels.put(
			new String("sysmodels.dnabiology.DNABiology"),
			new DNABiologyFactory());
	}
}
