/*
 * Created on Jun 18, 2003
 */
package sysmodels.standard;

import java.util.Vector;

import simulator.Cell;
import simulator.Simulation;
import simulator.SystemFactory;
import simulator.SystemModel;

/**
 * @author David Evans
 */

public class DisplaceModel extends StandardModel {
	static class SystemModelFactory extends SystemFactory {
		public SystemModel create(Simulation p_sim, String modelOptions) {
			DisplaceModel sb = new DisplaceModel(p_sim);
			Vector vec = new Vector();
			vec.add(modelOptions);
			sb.init(vec);
			return sb;
		}
	}

	static {
		Simulation.sysModels.put(
			new String("sysmodels.standard.DisplaceModel"),
			new SystemModelFactory());
	}

	protected boolean displaceCells(Cell cell1, Cell cell2) {
		boolean displaced = true;
		Cell cdOther;
		double dx = cell2.getX() - cell1.getX();
		double dy = cell2.getY() - cell1.getY();
		double dz = cell2.getZ() - cell1.getZ();
		Cell cell = cell2;

		while (displaced) {
			double x = cell.getX();
			double y = cell.getY();
			double z = cell.getZ();

			if (!locationInRange(x, y, z)) {
				getSimulation().writeErrorLine(
					"Cell location out of range: " + x + ", " + y + ", " + z);
				return false;
			} else {
				cdOther = getCell(x, y, z);
				if (cdOther == null) {
					displaced = false;
					setCell((int)x, (int)y, (int)z, cell);
				} else {
					displaced = true;
					cdOther.setPosition(x + dx, y + dy, z + dz);
					setCell((int)x, (int)y, (int)z, cell);
					cell = cdOther;
				}
			}
		}
		return true;
	}
	
	public DisplaceModel(Simulation p_sim) {
		super(p_sim);
	}
}
