/*
 * Created on Sep 18, 2003
 */
package test;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import util.Assert;

/**
 * @author David Evans
 */

public class TestAll {
	private static int numgood = 0;
	private static int numbad = 0;
	
	private static String[] parseParams(String s) {
		// Separates s into an array of String
		int numEls = 1;
		int index = 0;

		while ((index = s.indexOf((int) ' ', index)) >= 0) {
			numEls++;
			index++;
		}

		String[] res = new String[numEls];
		index = 0;
		int startindex = 0;
		int elemNo = 0;

		while ((index = s.indexOf((int) ' ', startindex)) >= 0) {
			Assert.check(elemNo < numEls);
			res[elemNo++] = s.substring(startindex, index);
			startindex = index + 1;
		}
		Assert.check(elemNo == numEls - 1);
		// Get the last string
		res[elemNo] = s.substring(startindex);
		return res;
	}

	private static String testOut = "test/test.out";
	private static String expectedDir = "test/expected";

	private static boolean diffFiles(String file1, String file2) {
		boolean diff = false;
		FileInputStream fin1 = null;
		FileInputStream fin2 = null;

		try {
			fin1 = new FileInputStream(file1);
		} catch (FileNotFoundException e) {
			System.err.println("Cannot open file: " + file1 + " - " + e);
		}

		try {
			fin2 = new FileInputStream(file2);
		} catch (FileNotFoundException e) {
			System.err.println("Cannot open file: " + file1 + " - " + e);
		}

		int in1;
		int lineno = 1;

		try {
			while ((in1 = fin1.read()) != -1) {
				while (in1 == '\r') {
					in1 = fin1.read (); 
				}
				
				int in2 = fin2.read();
				
				while (in2 == '\r') {
					in2 = fin2.read ();
				}
				
				if (in1 == in2) {
					if (in1 == '\n') {
						lineno++;
					}
				} else {
					diff = true;
					System.err.println(
						file1
							+ " / "
							+ file2
							+ ":"
							+ lineno
							+ ": "
							+ (char) in1
							+ " / "
							+ (char) in2);
				}
			}
		} catch (IOException e1) {
			System.err.println("Error reading file: " + e1);
			e1.printStackTrace();
		}
		return diff;
	}

	public static boolean runTest(String name, String args) {
		// Expected output is in test/expected/name.out
		System.err.print ("Testing " + name + "...");
		String testOut = "test/" + name + ".out";
		args += " -out:" + testOut + " -batch";
		System.err.println ("[ args: " + args + "]");
		// String [] test2 = parseParams ("-seed:5 input/mesh.cpa");

		String[] params = parseParams(args);
		try {
				gui.GUI.main(params);
		} catch (Exception e) {
			System.err.println ("Exception: " + e);
			e.printStackTrace();
			return false;
		}
		boolean ok = !diffFiles(testOut, expectedDir + "/" + name + ".out");
		
		if (ok) { 
			System.err.println ("ok");
			numgood++;
		} else {
			System.err.println ("ERROR");
			numbad++;
		}
		
		return ok;
	}

	public static void main(String[] args) {
		System.err.println("Running regression tests...");

		runTest("mesh1", "-seed:88 -steps:1 test/input/mesh.cpa -logall");
		runTest("mesh2", "-seed:89 -steps:10 test/input/mesh.cpa -logall");
		runTest("diffuse2", "-seed:93 -steps:8 test/input/diffuse2.cpa -logall");
		//! runTest("snake", "-seed:777 test/input/snake_chase.cpa test/input/chase.sim -nowarnundeclaredchemicals -steps:33 -logall");
		runTest("float", "-steps:8 test/input/diffuse2.cpa -seed:5 -system:sysmodels.floating.FloatingModel -logall");
		runTest("cylinder", "-seed:823 -nowarnundeclaredchemicals -steps:30 test/input/Circle.cpa test/input/Line.cpa test/input/Circle_Line.sim -logall");
		System.err.println("Tests Completed Successfully: " + numgood);
		if (numbad != 0) {
			System.err.println ("TESTS FAILED: " + numbad);
		}
				
				
	}
}
