/*
 * Created on Jul 2, 2003
 */
package util;

import java.util.Enumeration;
import java.util.Vector;

import simulator.SimRandom;

/**
 * @author David Evans
 */

public class DirectionSet {
	private Vector rep;

	public DirectionSet() {
		rep = null;
	}

	public DirectionSet(Direction dir) {
		rep = new Vector();
		add(dir);
	}

	public void add(Direction dir) {
		if (rep == null) {
			rep = new Vector();
		}

		rep.add(dir);
	}

	public Enumeration elements() {
		return rep.elements();
	}

	public boolean isEmpty() {
		return rep == null || rep.isEmpty();
	}

	public int size() {
		if (rep == null)
			return 0;
		return rep.size();
	}

	public Direction chooseRandomly() {
		Assert.check(!isEmpty());
		int index = SimRandom.nextInt(rep.size());
		// Report.message (0, "Choosing randomly: " + this + " => " + index + " = " + rep.elementAt (index));
		return (Direction) rep.elementAt(index);
	}

	public String toString() {
		String res = "{dirset: ";

		for (int i = 0; i < size(); i++) {
			Direction dir = (Direction) rep.elementAt(i);
			if (i != 0) {
				res += ", ";
			}
			res += dir;
		}

		res += "}";
		return res;
	}

	public boolean contains(Direction dir) {
		for (int i = 0; i < size(); i++) {
			Direction sd = (Direction) rep.elementAt(i);
			if (sd.equals(dir)) {
				return true;
			}
		}

		return false;
	}

	public void shuffle() {
		for (int i = 0; i < size(); i++) {
			// Swap the i'th element with a random element
			int sdir = SimRandom.nextInt(size());
			if (sdir != i) {
				Direction d1 = (Direction) rep.elementAt(i);
				Direction d2 = (Direction) rep.elementAt(sdir);
				rep.setElementAt(d1, sdir);
				rep.setElementAt(d2, i);
			}
		}
	}
}