/*
 * Created on Jul 12, 2003
 */
package util;

/**
 * @author David Evans
 */

public class Orientation {

	/**
	 * An Orientation is an angle in each direction.
	 */

	private double[] rot;
	private static final int XDIR = 0;
	private static final int YDIR = 1;
	private static final int ZDIR = 2;
	private static final int ndims = 3;

	public Orientation() {
		this(0.0, 0.0, 0.0);
	}

	// Values should be angles in radians	
	public Orientation(double xrot, double yrot, double zrot) {
		rot = new double[3];
		rot[0] = xrot;
		rot[1] = yrot;
		rot[2] = zrot;
	}

	public Orientation(Orientation orientation) {
		this(orientation.rot[0], orientation.rot[1], orientation.rot[2]);
	}

	public void add(Orientation o) {
		for (int i = 0; i < ndims; i++) {
			rot[i] = (rot[i] + o.rot[i]) % Math.toRadians (360);
		}
	}

	public void add(Position p) {
		rot[XDIR] =
			(rot[XDIR] + Math.toRadians(90 * p.getX())) % Math.toRadians(360);
		rot[YDIR] =
			rot[YDIR] + Math.toRadians(90 * p.getY()) % Math.toRadians(360);
		rot[ZDIR] =
			rot[ZDIR] + Math.toRadians(90 * p.getZ()) % Math.toRadians(360);
	}

	// Rotates the position pos according to this orientation
	public Position rotate(Position pos) {
		Position res;
		//		We rotate in each direction in order
		double cosx = Math.cos(rot[XDIR]);
		double sinx = Math.sin(rot[XDIR]);

		double[][] rotmatrix = { { 1, 0, 0, 0 }, {
				0, cosx, -sinx, 0 }, {
				0, sinx, cosx, 0 }, {
				0, 0, 0, 1 }
		};

		res = pos.rotate(rotmatrix);

		double cosy = Math.cos(rot[YDIR]);
		double siny = Math.sin(rot[YDIR]);

		double[][] rotmatrixy = { { cosy, 0, siny, 0 }, {
				0, 1, 0, 0 }, {
				-siny, 0, cosy, 0 }, {
				0, 0, 0, 1 }
		};

		res = res.rotate(rotmatrixy);

		double cosz = Math.cos(rot[ZDIR]);
		double sinz = Math.sin(rot[ZDIR]);

		double[][] rotmatrixz = { { cosz, -sinz, 0, 0 }, {
				sinz, cosz, 0, 0 }, {
				0, 0, 1, 0 }, {
				0, 0, 0, 1 }
		};

		res = res.rotate(rotmatrixz);
		return res;
	}

	public String toString() {
		return "<orientation: "
			+ Math.toDegrees(rot[0])
			+ ", "
			+ Math.toDegrees(rot[1])
			+ ", "
			+ Math.toDegrees(rot[2])
			+ ">";
	}
}
