package util;

import javax.vecmath.*;
import util.Direction;

public class Position {
	static private final Position ORIGIN = new Position(0.0, 0.0, 0.0);
	private double x = 0.0;
	private double y = 0.0;
	private double z = 0.0;

	public Position(Position p) {
		if (p != null) {
			x = p.getX();
			y = p.getY();
			z = p.getZ();
		}
	}

	public Position(double p_x, double p_y, double p_z) {
		x = p_x;
		y = p_y;
		z = p_z;
	}

	public Position offset(Direction dir) {
		return new Position(x + dir.getX(), y + dir.getY(), z + dir.getZ());
	}

	public double getX() {
		return x;
	}
	public double getY() {
		return y;
	}
	public double getZ() {
		return z;
	}

	public Vector3d getVector3dObject() {
		return new Vector3d(x, y, z);
	}

	public void setX(double p_x) {
		x = p_x;
	}
	public void setY(double p_y) {
		y = p_y;
	}
	public void setZ(double p_z) {
		z = p_z;
	}

	public double distance(Position p) {
		return Math.sqrt(
			(x - p.x) * (x - p.x)
				+ (y - p.y) * (y - p.y)
				+ (z - p.z) * (z - p.z));
	}

	public String toString() {
		return "(" + x + ", " + y + ", " + z + ")";
	}

	/**
	 * @param p1
	 * @param p2
	 * @return
	 */
	public boolean withinBlock(Position p1, Position p2) {
		return (
			x >= p1.x
				&& x <= p2.x
				&& y >= p1.y
				&& y <= p2.y
				&& z >= p1.z
				&& z <= p2.z);
	}

	// I'm sure this exists in a library somewhere...
	private double[] product(double[][] matrix, double[] vec) {
		double[] res = new double[vec.length];
		Assert.check(matrix.length == vec.length);

		for (int i = 0; i < vec.length; i++) {
			double val = 0.0;
			Assert.check(matrix[i].length == vec.length);
			for (int j = 0; j < vec.length; j++) {
				val += matrix[i][j] * vec[j];
			}

			res[i] = val;
		}

		return res;
	}

	public Position rotate(double[][] rotmatrix) {
		Assert.check(rotmatrix.length == 4);
		double length = distance(Position.ORIGIN);
		double[] hpos = { x / length, y / length, z / length, length };
		double[] rotpos = product(rotmatrix, hpos);
		// Report.message (0, "Rotating: " + this + " by " + Report.showMatrix (rotmatrix));
		// Report.message (0, "Homogenous position: " + Report.showArray (hpos));
		// Report.message (0, "Rotpos: " + Report.showArray (rotpos));		
		Position res =
			new Position(
				rotpos[0] * rotpos[3],
				rotpos[1] * rotpos[3],
				rotpos[2] * rotpos[3]);
		// Report.message (0, "New position: " + res);
		return res;
	}

	public Position scale(double d) {
		x = x * d;
		y = y * d;
		z = z * d;
		return this;
	}

	public Position add(Position position) {
		Position res =
			new Position(x + position.x, y + position.y, z + position.z);
		return res;
	}
}
