package util;

import java.util.*;

class Record {
	String key;
	Object value;

	public Record(String p_key, Object p_value) {
		key = p_key;
		value = p_value;
	}
}

public class StringTable {
	// A string-Object mapping.  Simple linear search is fine.
	Vector entries;

	public StringTable() {
		entries = new Vector();
	}

	public int size() {
		return entries.size();
	}

	public String getKey(int i) {
		return ((Record) entries.elementAt(i)).key;
	}

	public Object getValue(int i) {
		return ((Record) entries.elementAt(i)).value;
	}

	public Object lookup(String var) {
		for (Enumeration e = entries.elements(); e.hasMoreElements();) {
			Record el = (Record) e.nextElement();

			if (el.key.equals(var)) {
				return el.value;
			}
		}
		
		return null;
	}

	private void updateAdd (String key, Object value, boolean update) {
			// effects If key already exists, update its value to value.  If not, add key and value.
		for (Enumeration e = entries.elements(); e.hasMoreElements();) {
			Record el = (Record) e.nextElement();

			if (el.key.equals(key)) {
				el.value = value;
				return;
			}
		}

		if (update) {
			throw new RuntimeException ("Bad update: record not found: " + key);
		} else {		
			entries.add(new Record(key, value));
		}		
		
	}
	public void add (String key, Object value) {
		updateAdd (key, value, false);
	}

	public void update(String key, Object dir) {
		updateAdd (key, dir, true);		
	}

	public void setValue(String key, Object value) {
		// effects If key already exists, update its value to value.  If not, add key and value.
		for (Enumeration e = entries.elements(); e.hasMoreElements();) {
			Record el = (Record) e.nextElement();

			if (el.key.equals(key)) {
				el.value = value;
				return;
			}
		}

		entries.add(new Record(key, value));
	}

	public Enumeration keys() {
		Vector keys = new Vector();
		for (Enumeration e = entries.elements(); e.hasMoreElements();) {
			Record el = (Record) e.nextElement();
			keys.add(el.key);
		}
		return keys.elements();
	}

	public Enumeration values() {
		Vector values = new Vector();
		for (Enumeration e = entries.elements(); e.hasMoreElements();) {
			Record el = (Record) e.nextElement();
			values.add (el.value);
		}
		return values.elements();
	}

	public String toString() {
		String res = "";

		for (Enumeration e = entries.elements(); e.hasMoreElements();) {
			Record el = (Record) e.nextElement();
			res += el.key + ": " + el.value; // + "\n";
		}

		return res;
	}

	public String toCSVString() {
		String res = "";

		for (Enumeration e = entries.elements(); e.hasMoreElements();) {
			Record el = (Record) e.nextElement();
			res += el.value + ",";
		}

		res += "\n";
		return res;
	}

	public void addAll(StringTable table) {
		if (table != null) {
			for (Enumeration e = table.entries.elements(); e.hasMoreElements();) {
				Record el = (Record) e.nextElement();
				add (el.key, el.value);
			}				
		}
	}
}
