package ga;

import parsers.CPA.*;

public class Genome {
	//
	// Genome is immutable.
	// 
	// A Genome is a represented by a String consisted of 0 and 1 
	// The Genome is represented as: 
	// One Single State:
	//****************Plan 1****************116 bits per state
	//
	//// [xxxx        xxxx       xxxx        xxx    xxxxx xxx xxxxx xxx  xxxxx xxx   
	////  num_states  num_goals  state_name  color  emits                diffuses  
	////
	////         xx       xxxxx  xx   xx xxxx    x  xx       xxxxx  xx   xx xxxx    x  xx       xxxxx  xx   xx xxxx  
	////  trans  max/min  chem   dir  condition  &  max/min  chem   dir  condition  &  max/min  chem   dir  condition
	////
	////      xxxxx  xxxxxx xx  xxxxx  xxxxxx xx    xx        xx ]
	////  ->  chem   counter    chem   counter      oppo/same dir
	//
	//****************Plan 2***************210 bits per state
	//
	// 	Each number is a section of 10 bits
	//  [1           2      3     4                      
	//   state_name  color  emits diffuses  
	//
	//         5        6       7    8          9           10      11   12         13          14    15   16
	//  trans  max/min  chem    dir  condition  &  max/min  chem    dir  condition  &  max/min  chem  dir  condition 
	//
	//      17      18             19       20              21           ]
	//  ->  chem    counter        chem     counter         oppo/same dir
	//
	//****************Plan 3***************84 bits per state
	//
	//  [xxxxxx      xxx    xx xxxx  xxxx xx         
	//   state_name  color  emits    diffuses 
	// 
	//         xx       xxxxx   xx   xx xx      x  xx       xxxxx   xx   xx xx      x  xx       xxxxx  xx  xx xx 
	//  trans  max/min  chem    dir  condition  &  max/min  chem    dir  condition  &  max/min  chem  dir  condition 
	//
	//      xxxxx  xxxx     xxxxx  xxxx     xx        xx ]
	//  ->  chem   counter  chem   counter  oppo/same dir

	//
	// state A {
	//   emits (chem0-3, 0-2);
	//   transitions
	//      [chem0-3] [< | >] [0, 0.5, 1, 2] -> [ (state0-3) | (state0-3, state0-3) ]
	//      -> (A);
	//
	//  xx		chemical to emit
	//  xx      concentration
	//  xx xx xx x xx xx  transition
	// 15 bits per state, 2 states = 30 bits total

	private boolean gen[];
	private final static int STATE_LENGTH = 15;
	private int nstates;
	private int currentposition;
	
	public Genome(int nstates) {
		this.nstates = nstates;
		gen = new boolean[STATE_LENGTH * nstates];
		currentposition = 0;
		for (int i = 0; i < gen.length; i++)
			gen[i] = false;
	}

	public Genome(boolean p[]) {
		//		Assert.check(gen.length) == 130;
		int i = 0;
		while (p[i] == true || p[i] == false) {
			gen[i] = p[i];
		}
		currentposition = 0;
	}

	public boolean readNext () {
		return gen[currentposition++];
	}
	
	public int readNextInt (int bits) {
		int value = 0;
		int power = 1;
		
		for (int i = 0; i < bits; i++) {
			if (readNext ()) {
				value += power;
			}
			power *= 2;
		}
		
		return value;
	}
	
	public boolean getPosition(int i) {
		return gen[i];
	}

	public boolean[] getPosition(int i, int j) {
		boolean temp[] = {
		};
		int cnt = 0;
		for (int k = i; k <= j; k++) {
			temp[cnt] = gen[k];
			cnt++;
		}
		return temp;
	}

	public int tranSection(int i) {
		int tmp = 0;
		if (getPosition(i) == true)
			tmp += 2;
		if (getPosition(i + 1) == true)
			tmp += 1;
		return tmp;
	}

	public int numberOfStates() {
		return nstates;
	}

	public int getStatePosition(int i) {
		return i * STATE_LENGTH;
	}
}